package edu.stanford.hci.dtools.io.make;

import java.net.InetAddress;
import java.net.SocketException;
import java.util.Date;
import java.util.Timer;

import com.illposed.osc.OSCListener;
import com.illposed.osc.OSCMarshalIn;
import com.illposed.osc.OSCMessage;
import com.illposed.osc.OSCPortIn;
import com.illposed.osc.OSCPortOut;
import com.illposed.osc.utility.OSCPacketDispatcher;

/**
 * Poll a Make controller for a2d data
 * Then make it available using our "/adc" addressing scheme
 */
public class MakeControllerOSCPort implements OSCMarshalIn, OSCListener {

	private OSCPortIn in;
	private OSCPortOut out;
	private OSCPacketDispatcher dispatch;
	private MakeControllerPollAnalogInTask pollTask;
	private Timer timer;
	public MakeControllerOSCPort(int inPort, InetAddress outAddress, int outPort, int pollPeriodMs) {
		super();
		try {
			dispatch = new OSCPacketDispatcher();
			in = new OSCPortIn(inPort);
			in.addListener("/analogin/.*/value",this);
			in.startListening();
			
			out = new OSCPortOut(outAddress,outPort);
			pollTask = new MakeControllerPollAnalogInTask(out);
			timer = new Timer();
			timer.scheduleAtFixedRate(pollTask, 5, pollPeriodMs);
		} catch (SocketException e) {
			System.err.println("Caught Socket Exception while trying to connect to Make Controller.");
			e.printStackTrace();
		}
		
	}

	
	public void addListener(String address, OSCListener listener) {
		dispatch.addListener(address, listener);

	}

	public boolean changeListenerAddress(String oldAddress, String newAddress) {
		return dispatch.changeListenerAddress(oldAddress, newAddress);
	}

	public void close() {
		in.stopListening();
		in.close();
		out.close();
	}

	public OSCListener getListener(String address) {
		return dispatch.getListener(address);
	}

	public void removeAllListeners() {
		dispatch.removeAllListeners();

	}

	public boolean removeListener(String anAddress) {
		return dispatch.removeListener(anAddress);
	}

	public void acceptMessage(Date time, OSCMessage message) {
		if(message.getAddress().matches("/analogin/.*/value")) {
			
			//translate the address
			String num = message.getAddress().substring(10, 11);
			message.setAddress("/adc"+num);
			//and dispatch
			dispatch.dispatchPacket(message);
		}

	}

}
