package edu.stanford.hci.dtools.serial;
/***
 * WiTilt serial port adapter - listens to WiTilt v2.5 binary serial messages,
 * dispatches them as OSC packets.
 * TODO: * test
 * 		* do automatic setup upon startup (switch on binary mode and request data)
 * bjoern@stanford.edu, 04/2007
 */
import java.io.IOException;

import com.illposed.osc.OSCMarshalIn;
import com.illposed.osc.OSCMessage;

import gnu.io.SerialPort;
import gnu.io.SerialPortEvent;

public class WiTiltSerialPort extends DtoolsSerialPort implements OSCMarshalIn {

	private static final int RESET_TIMEOUT = 2000;
	private static final int RESET_CHAR = 0xE0;
	private int oscRxNextOp;
	private int oscRxMsgSize;
	private int oscRxReadBytes;
	private byte[] oscRxData;
	
	private int resetMode;
	private static final int WITILT_RXOP_WAITFORSTART=0;
	private static final int WITILT_RXOP_READMODE = 1;
	private static final int WITILT_RXOP_READBUFFER = 2;
	private static final int WITILT_RXOP_READEND = 3;
	
	private static final int RESET_NONE=100;
	private static final int RESET_1=101;
	private static final int RESET_2=102;
		
	public WiTiltSerialPort(String portName) {
 		super();
 		if(openCommPort(portName,57600, SerialPort.DATABITS_8,SerialPort.STOPBITS_1,SerialPort.PARITY_NONE)) {
 			readThread = new Thread(this);
 			readThread.start();
 		}
 	}
	
	
	public void serialEvent(SerialPortEvent event) {
		//System.out.println("SerialEvent came in.");
    	switch (event.getEventType()) {
    	case SerialPortEvent.BI:
    	case SerialPortEvent.OE:
    	case SerialPortEvent.FE:
    	case SerialPortEvent.PE:
    	case SerialPortEvent.CD:
    	case SerialPortEvent.CTS:
    	case SerialPortEvent.DSR:
    	case SerialPortEvent.RI:
    	case SerialPortEvent.OUTPUT_BUFFER_EMPTY:
    		break;
    	case SerialPortEvent.DATA_AVAILABLE:
    		try {
    			while (inputStream.available() > 0) {
    				buildOSCByteBuffer(inputStream.read());
    			}
    		} catch (IOException e) {
    			System.err.println("IOException in serialEvent!\n");
    		}
    		break;
    	}
		
	}
	
	/**
	 * Piece together an OSCByteBuffer one byte at a time.
	 * Expects OSC header 0xBE, size byte, and 8bit checksum.
	 * 
	 * @param c = current byte read from serial stream
	 */
    private void buildOSCByteBuffer(int c) {
    	if(c>256 )
    		System.err.println("WiTilt::buildOSCByteBuffer received too big argument: "+c);
    
    	if(c == RESET_CHAR) {
    		switch (resetMode) {
    		case RESET_NONE:
    			resetMode=RESET_1;
    			break;
    		case RESET_1:
    			resetMode=RESET_2;
    			handleReset();
    			resetMode=RESET_NONE;
    			return;
    		}
    	} else {
    		resetMode = RESET_NONE;
    	}
    	
    	
    	//System.out.print(new Character((char)c)+".");
    	switch(oscRxNextOp)
    	{
    	case WITILT_RXOP_WAITFORSTART:
    		if(c=='#') {
				oscRxNextOp = WITILT_RXOP_READMODE;
			}
    		break;
    	case WITILT_RXOP_READMODE:
    		if(c=='@') {
    			//data frame
    			oscRxMsgSize = 8;
    			oscRxReadBytes = 0;
				oscRxData = new byte[oscRxMsgSize+1];
				oscRxNextOp = WITILT_RXOP_READBUFFER;
    		} else if(c=='R') {
    			//ready
    			oscRxNextOp=WITILT_RXOP_WAITFORSTART;
    		}
    		break;
    	case WITILT_RXOP_READBUFFER:
    		try {
    		  oscRxData[oscRxReadBytes++] = (byte)c;
    		if(oscRxReadBytes == oscRxMsgSize) {
    		   oscRxData[oscRxReadBytes] = 0; //terminate string
    			oscRxNextOp = WITILT_RXOP_READEND;
    		}
    		} catch (java.lang.ArrayIndexOutOfBoundsException e) {
    			//something went wrong (i.e., got an erroneous 0xBE- throw the message away
    			oscRxNextOp = WITILT_RXOP_WAITFORSTART;
    		}
    		break;
    	case WITILT_RXOP_READEND:

    		   OSCMessage msgX = new OSCMessage();
    		   msgX.setAddress("/adc0");
    		   msgX.addArgument(new Integer(((int)oscRxData[2])*256+(0x000000FF & (int)oscRxData[3])));

    		   OSCMessage msgY = new OSCMessage();
    		   msgY.setAddress("/adc1");
    		   msgY.addArgument(new Integer(((int)oscRxData[4])*256+(0x000000FF & (int)oscRxData[5])));

    		   OSCMessage msgZ = new OSCMessage();
    		   msgZ.setAddress("/adc2");
    		   msgZ.addArgument(new Integer(((int)oscRxData[6])*256+(0x000000FF & (int)oscRxData[7])));

    		   OSCMessage msgXX = new OSCMessage();
    		   msgXX.setAddress("/adc3");
    		   msgXX.addArgument(new Integer(((int)oscRxData[2])*256+(0x000000FF & (int)oscRxData[3])));
				
    		   dispatcher.dispatchPacket(msgX);
			   dispatcher.dispatchPacket(msgY);
			   dispatcher.dispatchPacket(msgZ);
			   dispatcher.dispatchPacket(msgXX);
			   oscRxNextOp = WITILT_RXOP_WAITFORSTART;
				
			
    		break;
    	}
    }


	private void handleReset() {
		//create new osc message for reset
		OSCMessage msg = new OSCMessage();
		msg.setAddress("/reset");
		msg.addArgument(new Integer(1));
		//dispatch it
		dispatcher.dispatchPacket(msg);
		//sleep until the menu is on screen
		try { 
			Thread.sleep(RESET_TIMEOUT);
		} catch(Exception e) {
			e.printStackTrace();
		}
		//initialize
		initWiTilt();
		initParser();
	}


	@Override
	void initParser() {
		oscRxNextOp= WITILT_RXOP_WAITFORSTART;
		resetMode = RESET_NONE;
	}
	
	public void initWiTilt(){
		byte[] initSequence = {'1'}; //was 5 3 1 go through the menu - change mode -> binary -> start acquisition
		
		byte[] initSequence2 = {0x53}; //'S' = send data
		
		//send 5 3 1 S
		try {
			outputStream.write(initSequence );
			Thread.sleep(50);
			outputStream.write(initSequence2);
		} catch(Exception e) {
			System.err.println("Could not initialize WiTilt.");
		}
	}
    
  
}
