package edu.stanford.hci.exemplar;

import java.io.Serializable;
import java.util.List;

/**
 * Base class for Evaluators, which find events in data windows.
 *
 * Exemplar
 * Copyright (c) 2004-2007, Stanford University
 * All rights reserved.
 *
 * <p>
 * This software is distributed under the <a href="http://hci.stanford.edu/research/copyright.txt">
 * BSD License</a>.
 * </p>
 * 
 * 
 * @author <a href="http://bjoern.org/">Bjoern Hartmann</a> ( bjoern(AT)stanford.edu )
 * @author Leith Abdulla
 */
public interface EventEvaluator extends Serializable{
	/**
	 * Evaluate a time interval to recalculate matching regions
	 * @param curTime the end time of the evaluation window
	 * @param width  (curTime-width) is the starting time of the evaluation window
	 * @param fireEvents if true, this method should fire key events or dtools events; if false, just recalculate matches
	 * @return true if there was a match, false if there was not
	 */
	public boolean evaluate(int curTime, int width, boolean fireEvents);
	
	/**
	 * 
	 * @return minimum number of samples to look at
	 */
	public int getUpdateInterval();
	
	/**
	 * @return max width of samples to look at
	 */
	public int getUpdateWidthInterval();
	
	/**
	 * Learn a pattern based on a list of user-marked regions
	 * @param examples
	 */
	public void learn(List<EventRegion> examples);
	
	public void clear();
	/**
	 * Which type of events does this evaluator handle?
	 * @return
	 */
	public EventType getType();
	public String getPseudocode();
	
	/**
	 * Find "good" boundary points
	 * @param clutchRegion
	 * @return true if the region boundaries were adjusted during the call; false if it was not
	 */
	public boolean adjustRegion(EventRegion clutchRegion);
	
}
