package edu.stanford.hci.exemplar;

import java.io.Serializable;

import org.eclipse.swt.graphics.Point;
/**
 * Model class for a highlighted region in a sensor data visualization corresponding to a user event (user-created or detected) 
 *
 * Exemplar
 * Copyright (c) 2004-2007, Stanford University
 * All rights reserved.
 *
 * <p>
 * This software is distributed under the <a href="http://hci.stanford.edu/research/copyright.txt">
 * BSD License</a>.
 * </p>
 * 
 * 
 * @author <a href="http://bjoern.org/">Bjoern Hartmann</a> ( bjoern(AT)stanford.edu )
 */
public class EventRegion implements Serializable{
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	int startTime;
	int endTime;
	
	EventRegionOrigin origin = EventRegionOrigin.SYSTEM_CREATED;
	private String oscAddress;
	
	public EventRegion(int i, int j, String oscAddress) {
		startTime = i;
		endTime = j;
		checkAndSwap();
		this.oscAddress=oscAddress;
	}
	
	public EventRegion(EventRegion r) {
		this.startTime=r.startTime;
		this.endTime=r.endTime;
		this.oscAddress = r.getAddress();
	}

	private void checkAndSwap() {
		if(startTime>endTime) {
			int temp = startTime;
			startTime=endTime;
			endTime=temp;
		}
	}

	public int getEndTime() {
		return endTime;
	}
	public void setEndTime(int endTime) {
		this.endTime = endTime;
		checkAndSwap(); 
			
	}
	public int getStartTime() {
		return startTime;
	}
	public void setStartTime(int startTime) {
		this.startTime = startTime;
		checkAndSwap(); 
			
	}

	/* return the this range clipped against the visible range passed in, or null if it's not visible */
	public Point getVisibleRange(int from, int to) {
		//if this event is not in the currently visible range, return null
		if ((from > endTime) || (to < startTime)) {
			return null;
		} else {
			return new Point(Math.max(from, startTime),Math.min(to,endTime));
		}
		
	}

	public boolean isPointInRegion(int t) {
		return((startTime<=t)&&(endTime>=t));
	}

	public void move(Integer delta) {
		//prevent moving off left side of screen 
		startTime=Math.max(0, startTime+delta);
		endTime=Math.max(0, endTime+delta);
		
	}

	public EventRegionOrigin getOrigin() {
		return origin;
	}

	public void setOrigin(EventRegionOrigin origin) {
		this.origin = origin;
	}
	/**
	 * User created regions don't have an address here so they return null!
	 * @return
	 */
	public String getAddress() {
		return oscAddress;
	}
	public int getSize() {
		return endTime-startTime;
	}
}
