package edu.stanford.hci.exemplar;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
/**
 * Exemplar
 * Copyright (c) 2004-2007, Stanford University
 * All rights reserved.
 *
 * <p>
 * This software is distributed under the <a href="http://hci.stanford.edu/research/copyright.txt">
 * BSD License</a>.
 * </p>
 * 
 * 
 * @author <a href="http://bjoern.org/">Bjoern Hartmann</a> ( bjoern(AT)stanford.edu )
 * @author Leith Abdulla
 */
public class EventSet implements Serializable, PropertyChangeListener {
	
	
	private static final long serialVersionUID = 1L;
	private String oscAddress;
	public final List<Event> events = new ArrayList<Event>(); 
	private Event activeEvent = null;
	private final List<String> oscAddresses = new ArrayList<String>();
	private static int count = 0;

	private boolean modified=false;
	
	public EventSet(String oscAddress) {
		this.oscAddress =oscAddress;
		this.oscAddresses.clear();
		this.oscAddresses.add(oscAddress);
		Activator.getDefault().pcs.addPropertyChangeListener(ExemplarConstants.PROP_EVENTSET_MODIFIED, this);
	}

	public EventSet(List<String> oscAddresses) {
		this.oscAddresses.clear();
		this.oscAddresses.addAll(oscAddresses);
		this.oscAddress = oscAddresses.get(0); //HACK
	}
	
	public Event getActiveEvent() {
		return activeEvent;
	}
	public void setActiveEvent(Event e) {
		if(events.contains(e)) {
			if(activeEvent!=null) {
				activeEvent.setActive(false);
			}
			activeEvent = e;
			e.setActive(true);
			//notify others that the active event changed
			Activator.getDefault().pcs.firePropertyChange(ExemplarConstants.PROP_ACTIVE_EVENT_UPDATE,null,e);
			
		}
	}

	
	public String getOscAddress() {
		return oscAddress;
	}
	
	public List<String> getOscAddresses() {
		return oscAddresses;
	}

	//add an Event and return a ref to it
	public Event addEvent() {
		Event e;
		if(oscAddresses.size()==1) {
			e = new SingleSensorEvent(this);
		} else {
			e = new MultiSensorPickOneEvent(this);
			//e = new MultiSensorEvent(this);
		}
		events.add(e);
		e.setColor(ColorBrewer.getQualitative(count++));
		e.makeActive();
		Activator.getDefault().pcs.firePropertyChange(ExemplarConstants.PROP_ADD_EVENT, null, e);
		return e;
	}

	public void removeEvent(Event event) {
		events.remove(event);
		Activator.getDefault().pcs.firePropertyChange(ExemplarConstants.PROP_REMOVE_EVENT, null, event);
		if(activeEvent==event) {
			activeEvent = null;
		}
		//no more events
		if(events.size()==0) {
			setModified(false);
		}
	}

	public List<Event> getEvents() {
		return events;
	}

	/**
	 * Write object out to file
	 * @param out
	 * @throws IOException
	 */
	private void writeObject(ObjectOutputStream out) throws IOException {
		out.defaultWriteObject();
	}
	
	/**
	 * Read object back in from file and reregister with the propertychangesupport.
	 * 	@param in
	 * @throws IOException
	 * @throws ClassNotFoundException
	 */
	private void readObject(ObjectInputStream in) throws IOException, ClassNotFoundException {
		
		in.defaultReadObject();
		//Notify widgets (and all other listeners) that the old event set is stale
		Activator.getDefault().pcs.firePropertyChange(ExemplarConstants.PROP_REPLACE_EVENT_SET_MODEL,oscAddresses,this);
	}

	public boolean hasBeenModified() {
		return modified;
	}

	public void setModified(boolean modified) {
		this.modified = modified;
	}

	public void propertyChange(PropertyChangeEvent evt) {
		if(ExemplarConstants.PROP_EVENTSET_MODIFIED.equals(evt.getPropertyName())) {
			setModified(true);
		}
		
	}

	public String getOscAddressesAsString() {
		String s = "";
		for(String address:getOscAddresses()) {
			s+=address+" ";
		}
		s = s.trim();
		return s;
	}
}
