package edu.stanford.hci.exemplar;

import java.util.ArrayList;
import java.util.List;
/**
 * Exemplar
 * Copyright (c) 2004-2007, Stanford University
 * All rights reserved.
 *
 * <p>
 * This software is distributed under the <a href="http://hci.stanford.edu/research/copyright.txt">
 * BSD License</a>.
 * </p>
 * 
 * 
 * @author <a href="http://bjoern.org/">Bjoern Hartmann</a> ( bjoern(AT)stanford.edu )
 * @author Leith Abdulla
 */
public class SingleSensorEvent extends Event{
	
	private static final long serialVersionUID = 1L;
	protected EventEvaluator evaluator = null;
	protected int lastTime =0;

	
	public SingleSensorEvent(EventSet parent) {
		super(parent);
		//	SEBUG 
		evaluator = new ThresholdEventEvaluator(this,parent.getOscAddress());
	}

	
	public SingleSensorEvent(SingleSensorEvent copy) {
		super(copy);
		this.evaluator =copy.evaluator;
		this.lastTime = copy.lastTime;
	}


	@Override
	public void recalculate(boolean relearn) {
		
		Activator.getDefault().saveTimers();
		Activator.getDefault().pauseAll();
				//throw out all system-created regions
				List<EventRegion> keepers = new ArrayList<EventRegion>();
				for(EventRegion r:getRegions()) {
					if (r.getOrigin().equals(EventRegionOrigin.USER_CREATED)) {
						keepers.add(r);
					}
				}
				getRegions().clear();
				getRegions().addAll(keepers);
				//if(keepers.size()>0) {
					if(relearn) {
						evaluator.learn(keepers);
					}
					//recreate system regions based on user regions
	
					if(this.getEventSet().getActiveEvent().type.equals(EventType.THRESHOLD))
					{
						evaluator.evaluate(Activator.getDefault().getCurrentTime()-1,Activator.getDefault().getCurrentTime()-1,false);
					}
					else if(this.getEventSet().getActiveEvent().type.equals(EventType.PATTERN))
					{
						//don't re-evaluate in pattern mode for now
					}
				//}
				//we're probably not in the UI thread, so notify our Widget observers with the async thread switch workaround
				Activator.getDefault().getWorkbench().getDisplay().asyncExec (new Runnable () {
					public void run () {
						notifyObservers();
					}
				});
		Activator.getDefault().restoreTimers();
	}

	@Override
	public void update() {
				if(evaluator!=null) {
					int curTime = Activator.getDefault().getCurrentTime()-2; // this is giving us problems - data not yet available
					int width = (curTime-lastTime);
					//if the time passed since we last evaluated exceeds the evaluator's threshold
					if(width>=evaluator.getUpdateInterval()) {
						if(evaluator instanceof ThresholdEventEvaluator) {
							evaluator.evaluate(curTime,width,true);
							lastTime = curTime;
						} else { //PATTERN
							PatternEventEvaluator pee = (PatternEventEvaluator)evaluator;
							boolean match = evaluator.evaluate(curTime,pee.getPatternSize(),true);
							if(match) {
								lastTime = curTime+pee.getPatternSize();
							} else {
								lastTime = curTime;
							}
								
							
						}
						
						
					}
				}
	}

	@Override
	public void evaluatorNotify(EventEvaluator evaluator, boolean newState) {
		if(doOutput) {
			if(newState) {
				Activator.getDefault().pcs.firePropertyChange(ExemplarConstants.PROP_EVENT_ON, null, key);
			} else {
				Activator.getDefault().pcs.firePropertyChange(ExemplarConstants.PROP_EVENT_OFF, null, key);
			}
		}
	}

	@Override
	public void setType(EventType type, String string)  {
		// if we're not changing event type, do nothing 
		// (i.e., don't trash the evaluator we have that has learned state) 
		if(this.type.equals(type))
			return;
		
		this.type = type;
		if(type.equals(EventType.THRESHOLD)) {
			evaluator = new ThresholdEventEvaluator(this,string);
		} else if (type.equals(EventType.PATTERN)) {
			evaluator = new PatternEventEvaluator(this,string);
		} else {
			evaluator = null;
		}
	}


	@Override
	public EventEvaluator getEvaluator(String OscAddress) {
		return evaluator;
	}


	@Override
	public EventType getType(String string) {
		return type;
	}


	@Override
	public String getPseudocode() {
		String s = "//output for Event "+this.getName()+"\n";
		s+="//was hooked to sensor(s) "+this.getEventSet().getOscAddressesAsString()+"\n\n";
		s+=evaluator.getPseudocode();
		return s;
	}


	@Override
	public void adjustRegion(EventRegion clutchRegion) {
		evaluator.adjustRegion(clutchRegion);
	}

}
