package edu.stanford.hci.exemplar.exensions;
import edu.stanford.hci.exemplar.Sensor;
/**
 * Abstract base class for user-defined filters (aka FilterExtensions) in Exemplar.
 * Users have to override the process, getName, and getDescription methods.
 * To retrieve input data, they should use the provided getInput() method.
 *
 * Copyright (c) 2004-2007, Stanford University
 * All rights reserved.
 *
 * <p>
 * This software is distributed under the <a href="http://hci.stanford.edu/research/copyright.txt">
 * BSD License</a>.
 * </p>
 * 
 * 
 * @author <a href="http://bjoern.org/">Bjoern Hartmann</a> ( bjoern(AT)stanford.edu )
 * @author Leith Abdulla
 */
public abstract class FilterExtension {
	private Sensor sensor;
	private int level;

	/**
	 * Perform the filter calculation.
	 * @param time the current timestamp
	 * @return a float between 0.f and 1.f that is the filtered value for the given time
	 */
	public abstract Float process(int time);
	
	/**
	 * The string returned by this method will be shown in the title of the 
	 * filter widget in the Exemplar UI. 
	 * @return name of this filter  
	 */
	public abstract String getName();
	/**
	 * The string returned by this method will be shown in the body of the 
	 * filter widget in the Exemplar UI.
	 * @return one-line description of this filter
	 */
	public abstract String getDescription();
	
	/**
	 * Internal utility function.
	 * @param s
	 */
	public final void setSensor(Sensor s) {
		this.sensor =s;
	}
	/**
	 * Internal utility function.
	 * @param l
	 */
	public final void setLevel(int l) {
		this.level = l;
	}
	/**
	 * Get sensor input data for a given time.
	 * @param time
	 * @return sensor data (should be between 0.f and 1.f, but no guarantees)
	 */
	protected Float getInput(int time) {
		return sensor.getFilteredValueforTime(time, level-1);
	}
}
