package edu.stanford.hci.exemplar.exensions;

import java.io.File;
import java.io.FilenameFilter;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import edu.stanford.hci.exemplar.ExemplarConstants;
import edu.stanford.hci.exemplar.Sensor;
/**
 * Exemplar
 * Copyright (c) 2004-2007, Stanford University
 * All rights reserved.
 *
 * <p>
 * This software is distributed under the <a href="http://hci.stanford.edu/research/copyright.txt">
 * BSD License</a>.
 * </p>
 * 
 * 
 * @author <a href="http://bjoern.org/">Bjoern Hartmann</a> ( bjoern(AT)stanford.edu )
 * @author Leith Abdulla
 */
public class FilterExtensionLoader {
	private class FilterExtensionPair {
		public Sensor sensor;
		public Class theClass;
		public FilterExtensionPair(Sensor sensor, Class theClass) {
			this.sensor=sensor;
			this.theClass=theClass;
		}
	}
	private List<Class> filterList = new ArrayList<Class>();
	private static FilterExtensionLoader instance=null;
	private Map<FilterExtensionPair,FilterExtension> map = new HashMap<FilterExtensionPair,FilterExtension>();
	private ClassLoader parentClassLoader = Thread.currentThread().getContextClassLoader();
	private ClassLoader classLoader;
	public static FilterExtensionLoader getInstance() {
		if( instance==null) {
			instance = new FilterExtensionLoader();
			instance.load();
		}
		return instance;
	}
	
	public void load()  {
		filterList.clear();
		map.clear();
		//build URL list
		URL[] urls = new URL[1];
		try {
			urls[0] = new URL("file:"+ExemplarConstants.filterExtensionPath);
		} catch (MalformedURLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		classLoader = new URLClassLoader(urls,parentClassLoader);
		//open up the directory where filter class files are stored
		//get all matching files
		//for all matches:
		//http://javaalmanac.com/egs/java.io/GetFiles.html
		
		File dir = new File(ExemplarConstants.filterExtensionPath);
		
		
		FilenameFilter filter = new FilenameFilter() {
	        public boolean accept(File dir, String name) {
	            return name.endsWith(".class");
	        }
	    };
	    
	    String[] children = dir.list(filter); //null if dir doesn't exist of IO Error occurs
	    if(!dir.isDirectory() || children==null) {
	    	return;// if this directory is no good, return.	
	    }
	    
	    for(String child:children) {
	    	String prefix = child.substring(0,child.lastIndexOf(".class"));
	    	try {
				Class cls = Class.forName(prefix,true, classLoader);
				if (cls.getSuperclass().equals(FilterExtension.class)) {
					filterList.add(cls);
				}
			} catch (ClassNotFoundException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
	    }
	    /*
		Enumeration<URL> en;
		try {
			en = classLoader.getResources("*.class");
			while(en.hasMoreElements()) 
			{
				URL url = en.nextElement();
				System.err.println("found: "+url);
			}
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		*/
	}

	public FilterExtension getFilter(Sensor sensor, Class detail) {
		FilterExtensionPair p =new FilterExtensionPair(sensor,detail); 
		if(map.containsKey(p)) {
			return map.get(p);
		} else {
			try {
				FilterExtension e = (FilterExtension) detail.newInstance();
				map.put(p,e);
				return e;
			} catch (Exception e) {
				System.err.println("FilterExtension: Caught exception when trying to load a custom Filter.");
				return null;
			}
		}
	}

	public List<Class> getFilterList() {
		return filterList;
	}
}
