package edu.stanford.hci.exemplar.output;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.net.InetAddress;

import com.illposed.osc.OSCMessage;
import com.illposed.osc.OSCPortOut;

import edu.stanford.hci.exemplar.Activator;
import edu.stanford.hci.exemplar.ExemplarConstants;
import edu.stanford.hci.exemplar.preferences.PreferenceConstants;
/**
 * Listens for PROP_EVENT_ON and PROP_EVENT_OFF messages and relays them as 
 * d.tools-compatible OSC messages over UDP
 *
 * Exemplar
 * Copyright (c) 2004-2007, Stanford University
 * All rights reserved.
 *
 * <p>
 * This software is distributed under the <a href="http://hci.stanford.edu/research/copyright.txt">
 * BSD License</a>.
 * </p>
 * 
 * 
 * @author <a href="http://bjoern.org/">Bjoern Hartmann</a> ( bjoern(AT)stanford.edu )
 * @author Leith Abdulla
 */
public class OscOutput implements PropertyChangeListener {
	private OSCPortOut port;
	
	public OscOutput() {
		super();
		
		try {
			//InetAddress address = InetAddress.getLocalHost();
			InetAddress address = InetAddress.getByName(Activator.getDefault().getPreferenceStore().getString(PreferenceConstants.P_UDP_OUTPUT_IPADDRESS));
			port = new OSCPortOut(address,Activator.getDefault().getPreferenceStore().getInt(PreferenceConstants.P_UDP_OUTPUT_PORT));
		} catch (Exception e) {
			port =null;
		}
		Activator.getDefault().pcs.addPropertyChangeListener(ExemplarConstants.PROP_EVENT_ON, this);
		Activator.getDefault().pcs.addPropertyChangeListener(ExemplarConstants.PROP_EVENT_OFF, this);
		Activator.getDefault().pcs.addPropertyChangeListener(ExemplarConstants.PROP_NEW_FILTERED_DATA, this);
	}

	public void propertyChange(PropertyChangeEvent evt) {
		if(port!=null) {
			OSCMessage msg =new OSCMessage();
			msg.setAddress((String)evt.getNewValue());
			
			if(ExemplarConstants.PROP_EVENT_ON.equals(evt.getPropertyName())) {
				System.out.println("Sending ON OSC Message at "+System.currentTimeMillis()+"to: "+evt.getNewValue());
				msg.addArgument(new Integer(1));
				try{
					port.send(msg);
				} catch (Exception e) {
					System.err.println("OscOutput: Couldn't send osc udp port message");
				}
			} else if(ExemplarConstants.PROP_EVENT_OFF.equals(evt.getPropertyName())) {
				System.err.println("Sending OFF OSC Message at "+System.currentTimeMillis()+"to: "+evt.getNewValue());
				msg.addArgument(new Integer(0));
				try{
					port.send(msg);
				} catch (Exception e) {
					System.err.println("OscOutput: Couldn't send osc udp port message");
				}
			} else if(ExemplarConstants.PROP_NEW_FILTERED_DATA.equals(evt.getPropertyName())) {
				try{
					msg.addArgument((Integer)evt.getOldValue());
					port.send(msg);
				} catch (Exception e) {
					System.err.println("OscOutput: Couldn't send osc udp port message");
				}
			}
			
		}
	}

}
