package edu.stanford.hci.exemplar.output;

import java.awt.Robot;
import java.awt.event.KeyEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import edu.stanford.hci.exemplar.Activator;
import edu.stanford.hci.exemplar.ExemplarConstants;
/**
 * Exemplar
 * Copyright (c) 2004-2007, Stanford University
 * All rights reserved.
 *
 * <p>
 * This software is distributed under the <a href="http://hci.stanford.edu/research/copyright.txt">
 * BSD License</a>.
 * </p>
 * 
 * 
 * @author <a href="http://bjoern.org/">Bjoern Hartmann</a> ( bjoern(AT)stanford.edu )
 * @author Leith Abdulla
 */
public class RobotOutput implements PropertyChangeListener {
	private Robot r;
	private boolean active;
	public RobotOutput() {
		super();
		Activator.getDefault().pcs.addPropertyChangeListener(ExemplarConstants.PROP_EVENT_ON, this);
		Activator.getDefault().pcs.addPropertyChangeListener(ExemplarConstants.PROP_EVENT_OFF, this);
		try {
			 r = new Robot();
		} catch (Exception e) {}
		
		active = false; //change to start sending out key presses
		
	}

	public void propertyChange(PropertyChangeEvent evt) {
		if(active) {
			int keyCode = getKeyForString((String)evt.getNewValue());
			
			if(ExemplarConstants.PROP_EVENT_ON.equals(evt.getPropertyName())) {
				try {
					r.keyPress(keyCode);
				} catch(Exception e){
					System.err.println("RobotOutput failed: "+e);
				} finally {
					
				}
				
			} else if(ExemplarConstants.PROP_EVENT_OFF.equals(evt.getPropertyName())) {
				try {
					r.keyPress(keyCode);
				} catch(Exception e){
					
				} finally {
					
				}
			}
		}
	}

	private int getKeyForString(String string) {
		string = string.toUpperCase();
		if(string.length()==1) {
			//return key code for numbers and keys here
			char c = string.charAt(0);
			if((c>=0x30 && c<=39) || (c>=0x41 && c<=0x5A)) {
				//if key is 0-9 or A-Z
				//see http://www.flex-compiler.lcs.mit.edu/jdk/api/java.awt.event.KeyEvent.html
				return c; 
			}
		}	
		else {
			//handle "Up","Down", etc.
			if(string.equals("UP")) {
				return KeyEvent.VK_UP;
			} else if(string.equals("DOWN")) {
				return KeyEvent.VK_DOWN;
			} else if(string.equals("RIGHT")) {
				return KeyEvent.VK_RIGHT;
			} else if(string.equals("LEFT")) {
				return KeyEvent.VK_LEFT;
			} else if (string.equals("ENTER")) {
				return KeyEvent.VK_ENTER;
			} else if (string.equals("SPACE")) {
				return KeyEvent.VK_SPACE;
			} 

		}
		// if we couldn't parse, 
		return KeyEvent.VK_UNDEFINED;
	}

	public boolean isActive() {
		return active;
	}

	public void setActive(boolean active) {
		this.active = active;
	}
}
