package edu.stanford.hci.exemplar;
/**
 * Model class for a big sensor visualization
 *
 * Exemplar
 * Copyright (c) 2004-2007, Stanford University
 * All rights reserved.
 *
 * <p>
 * This software is distributed under the <a href="http://hci.stanford.edu/research/copyright.txt">
 * BSD License</a>.
 * </p>
 * 
 * 
 * @author <a href="http://bjoern.org/">Bjoern Hartmann</a> ( bjoern(AT)stanford.edu )
 * @author Leith Abdulla
 * 
 */
public class BigViewModel {
	private static final int MIN_WIDTH = 100;
	private static final int CANVAS_WIDTH = 600;
	private static final int NEUTRAL_ZOOM = 80;
	int startTime; //start time of the visualization
	int endTime; //end time of the visualization
	Integer zoom;
	int position; //0..number of samples
	boolean playing; //playing or paused?
	Sensor sensor;
	
	
	public BigViewModel() {
		super();
		startTime=0;
		endTime=1;
		zoom=1;
		position=1;
		playing=true;
		
	}
	public int getBufferSize() {
		return sensor.getBufferSize();
	}
	public int getEndTime() {
		
		return endTime;
	}
	public void setEndTime(int endTime) {
		this.endTime = endTime;
	}
	public boolean isPlaying() {
		return playing;
	}
	public void setPlaying(boolean playing) {
		this.playing = playing;
	}
	
	public int getZoom() {
		return zoom;
	}
	public void setZoom(int zoom) {
		this.zoom = zoom;
		computeCenteredStartEndTime();
	}
	public Sensor getSensor() {
		return sensor;
	}
	public void setSensor(Sensor sensor) {
		this.sensor = sensor;
	}
	public int getStartTime() {
		return startTime;
	}
	public void setStartTime(int startTime) {
		startTime = Math.max(0,startTime);
		int delta =startTime- this.startTime;
		this.startTime = startTime;
		this.endTime = Math.min(getBufferSize(),endTime+delta);
	}
	/**
	 * recompute start and end time, but keep the center of the view where it currently is
	 *
	 */
	
	public void computeCenteredStartEndTime() {
		int center = startTime+ (endTime-startTime)/2;
		int delta = computeDelta();
		startTime = Math.max(0, center-delta/2);
		endTime = Math.min(getBufferSize(), center+(delta-delta/2));
	}
	
	public void computeStartTime() {
		//keep end time fixed; recompute start time
		//show between MIN_WIDTH and sensor.getBufferSize samples
		int delta = computeDelta();
		if(delta>(endTime-1))
			delta = endTime-1;
		startTime = endTime-delta;
	}
	
	private int computeDelta() {
		int delta;
		
		if(zoom >=NEUTRAL_ZOOM) {
			//zoomed in
			delta = MIN_WIDTH + ((Float)(((100.f-zoom.floatValue())/20.f)*((float)(CANVAS_WIDTH-MIN_WIDTH)))).intValue();
		} else {
			delta = CANVAS_WIDTH + ((Float)(((80.f-zoom.floatValue())/80.f)*((Integer)(sensor.getBufferSize()-CANVAS_WIDTH)).floatValue())).intValue();
		}
		return delta;
	}

	public int computeZoom() {
		int delta = endTime-startTime;
		//these are the inverse of the calculations above in computeDelta
		if(delta<=CANVAS_WIDTH) {
			zoom =(int)((-20.f)*((float)(delta+4*MIN_WIDTH-5*CANVAS_WIDTH))/((float)(CANVAS_WIDTH-MIN_WIDTH)));
		} else {
			zoom =(int)( 80.f*((float)delta-sensor.getBufferSize())/((float)(-1)*sensor.getBufferSize()+CANVAS_WIDTH));
		}
		
		return zoom;
	}
	
	public void computeEndTime() {
		endTime = getBufferSize();
		
	}
	public void computePlayingEndTime() {
		//snap to playing end if close
		//if(getBufferSize()-endTime<10) {
			endTime = getBufferSize();
			computeStartTime();
		//}
		//otherwise leave unchanged
		
	}
	public String getOscAddress() {
		
		return sensor.getOSCAddress();
	}
	public float getDelta() {
		return computeDelta();
	}
	
	
}
