package edu.stanford.hci.exemplar;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
/**
 * Exemplar
 * Copyright (c) 2004-2007, Stanford University
 * All rights reserved.
 *
 * <p>
 * This software is distributed under the <a href="http://hci.stanford.edu/research/copyright.txt">
 * BSD License</a>.
 * </p>
 * 
 * 
 * @author <a href="http://bjoern.org/">Bjoern Hartmann</a> ( bjoern(AT)stanford.edu )
 * @author Leith Abdulla
 */
public class MultiSensorPickOneEvent extends MultiSensorEvent {
	
	private static final long serialVersionUID = 1L;
	String pickedSensor=null;
	
	
	public MultiSensorPickOneEvent(EventSet parent) {
		super(parent);
	}
	
	/**
	 * Pick the best sensor out of the bunch
	 * @param clutchRegion
	 */
	public void pickSensor(EventRegion clutchRegion) {
		float maxPwr = Float.MIN_VALUE;
		String maxAddress = null;
		//find the single sensor with the largest power in the signal for the clutch region
		for(Map.Entry<String,EventEvaluator> entry: oscToEvalMap.entrySet()) {
			Sensor sensor = Activator.getDefault().getSensorForOscAddress(entry.getKey());
			float pwr = sensor.getPowerForTimeSlice(clutchRegion.startTime,clutchRegion.endTime);
			
			if(pwr>maxPwr) {
				maxPwr = pwr;
				maxAddress = entry.getKey();
			}
		}
		pickedSensor = maxAddress;
		Activator.getDefault().pcs.firePropertyChange(ExemplarConstants.PROP_PICKED_SENSOR, null, maxAddress);
	}

	@Override
	public void adjustRegion(EventRegion clutchRegion) {
		pickSensor(clutchRegion);
		if(pickedSensor!=null) {
			EventEvaluator evaluator = oscToEvalMap.get(pickedSensor);
				evaluator.adjustRegion(clutchRegion);
		}
	}
	
	@Override
	public void recalculate(boolean relearn) {
	
		if(pickedSensor==null)
			return;
		
		EventEvaluator evaluator = oscToEvalMap.get(pickedSensor);
		Activator.getDefault().saveTimers();
		Activator.getDefault().pauseAll();
				//throw out all system-created regions
				List<EventRegion> keepers = new ArrayList<EventRegion>();
				for(EventRegion r:getRegions()) {
					if (r.getOrigin().equals(EventRegionOrigin.USER_CREATED)) {
						keepers.add(r);
					}
				}
				getRegions().clear();
				getRegions().addAll(keepers);
				//if(keepers.size()>0) {
					if(relearn) {
						evaluator.learn(keepers);
					}
					//recreate system regions based on user regions
	
					if(this.getEventSet().getActiveEvent().type.equals(EventType.THRESHOLD))
					{
						evaluator.evaluate(Activator.getDefault().getCurrentTime()-1,Activator.getDefault().getCurrentTime()-1,false);
					}
					else if(this.getEventSet().getActiveEvent().type.equals(EventType.PATTERN))
					{
						//don't re-evaluate in pattern mode for now
					}
				//}
				//we're probably not in the UI thread, so notify our Widget observers with the async thread switch workaround
				Activator.getDefault().getWorkbench().getDisplay().asyncExec (new Runnable () {
					public void run () {
						notifyObservers();
					}
				});
		Activator.getDefault().restoreTimers();
	}

	@Override
	public void update() {
		if(pickedSensor==null)
			return;
		
		EventEvaluator evaluator = oscToEvalMap.get(pickedSensor);
				if(evaluator!=null) {
					Integer lastTime = lastTimeMap.get(evaluator);
					int curTime = Activator.getDefault().getCurrentTime()-2; // this is giving us problems - data not yet available
					int width = (curTime-lastTime);
					//if the time passed since we last evaluated exceeds the evaluator's threshold
					if(width>=evaluator.getUpdateInterval()) {
						if(evaluator instanceof ThresholdEventEvaluator) {
							evaluator.evaluate(curTime,width,true);
							lastTime = curTime;
						} else { //PATTERN
							PatternEventEvaluator pee = (PatternEventEvaluator)evaluator;
							boolean match = evaluator.evaluate(curTime,pee.getPatternSize(),true);
							if(match) {
								lastTime = curTime+pee.getPatternSize();
							} else {
								lastTime = curTime;
							}
								
							
						}
						
						
					}
				 lastTimeMap.remove(evaluator);
				 lastTimeMap.put(evaluator, lastTime);
				}
	}

	@Override
	public void evaluatorNotify(EventEvaluator evaluator, boolean newState) {
		if(doOutput) {
			if(newState) {
				Activator.getDefault().pcs.firePropertyChange(ExemplarConstants.PROP_EVENT_ON, null, key);
			} else {
				Activator.getDefault().pcs.firePropertyChange(ExemplarConstants.PROP_EVENT_OFF, null, key);
			}
		}
	}

}
