package edu.stanford.hci.exemplar;

import java.util.Random;
import java.util.Vector;
/**
 * Exemplar
 * Copyright (c) 2004-2007, Stanford University
 * All rights reserved.
 *
 * <p>
 * This software is distributed under the <a href="http://hci.stanford.edu/research/copyright.txt">
 * BSD License</a>.
 * </p>
 * 
 * 
 * @author <a href="http://bjoern.org/">Bjoern Hartmann</a> ( bjoern(AT)stanford.edu )
 * @author Leith Abdulla
 */
public class RingBuffer {

	Vector<Float> v;
	int size;
	int index; //always points at the current head of the queue
	public RingBuffer(int i) {
		super();
		size = i;
		init();
		randomize();
	}

	private void init() {
		v = new Vector<Float>(size);
		//initialize buffer to 0
		for(int j=0; j<size;j++) {
			v.add(j,0.f);
		}
		index = size-1;
	}

	// add a value at the head of the queue
	public void add(Float value) {
		index = ((index+1)%size); //move head index ahead by one (mod size)
		v.set(index,value);
	}
	
	// fill the buffer with random data
	private void randomize() {
		Random rnd = new Random();
		for(int j=0; j<size;j++) {
			v.set(j,rnd.nextFloat());
		}

	}

	//get(0) returns last added item
	//get(1) returns item added next-to-last
	//get(size-1) returns oldest item
	public float get(int i) {
		int temp = ((index - i + size)%size);
		return v.get(temp);
	}
	public float getHead() {
		return get(0);
	}
	
	public int getSize() {
		return size;
	}
	
	// change size of buffer (reinitializes the buffer)
	public void setSize(int n) {
		size=n;
		init();
	}
	
	
}
