package edu.stanford.hci.exemplar;

import java.util.Date;
import java.util.TimerTask;
import java.util.Vector;

import com.illposed.osc.OSCListener;
import com.illposed.osc.OSCMessage;
/**
 * Exemplar
 * Copyright (c) 2004-2007, Stanford University
 * All rights reserved.
 *
 * <p>
 * This software is distributed under the <a href="http://hci.stanford.edu/research/copyright.txt">
 * BSD License</a>.
 * </p>
 * 
 * 
 * @author <a href="http://bjoern.org/">Bjoern Hartmann</a> ( bjoern(AT)stanford.edu )
 * @author Leith Abdulla
 */
public class UpdateDataTask extends TimerTask implements OSCListener{
	Vector<Float> currentValues = new Vector<Float>(ExemplarConstants.NUM_SENSORS);
	Vector<String> propNames = new Vector<String>(ExemplarConstants.NUM_SENSORS);
	boolean run=true;
	
	public void pause() {
		run=false;
	}
	public void resume() {
		run=true;
	}
	public UpdateDataTask() {
		super();
		//init array to zero
		for(int i=0; i<ExemplarConstants.NUM_SENSORS; i++) {
			currentValues.add(i,0.f);
			propNames.add("/adc".concat(Integer.toString(i)));
		}
	}

	@Override
	// notify all listeners of the current values of all sensors
	public void run() {
		if(run) {
			for(int i=0; i<ExemplarConstants.NUM_SENSORS; i++) {
				Activator.getDefault().pcs.firePropertyChange(propNames.get(i), null, currentValues.get(i));
			}
		}
	}

	// receive a OSC hardware message and store the value
	public void acceptMessage(Date arg0, OSCMessage msg) {
		if(msg.getAddress().startsWith("/adc")) {
			int index =Integer.parseInt(msg.getAddress().substring(4)); 
			if((0<=index) && ExemplarConstants.NUM_SENSORS>index) {
				//convert data into 0.f - 1.f range
				float val = Converter.a2dIntToFloat((Integer)(msg.getArguments()[0]));
				//save data
				currentValues.set(index, val);
			}
		}
		// annotate via clutch button
		else if (msg.getAddress().startsWith("/in")) {
			Activator.getDefault().pcs.firePropertyChange(ExemplarConstants.PROP_CLUTCH,null,(Integer)msg.getArguments()[0]);
		}
		
	}
	public boolean isPaused() {
		return !run;
	}
	public void setPaused(boolean paused) {
		run = !paused;
		
	}
}
