package edu.stanford.hci.exemplar.preferences;

import org.eclipse.jface.preference.FieldEditorPreferencePage;
import org.eclipse.jface.preference.IntegerFieldEditor;
import org.eclipse.jface.preference.RadioGroupFieldEditor;
import org.eclipse.jface.preference.StringFieldEditor;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPreferencePage;

import edu.stanford.hci.exemplar.Activator;
import edu.stanford.hci.exemplar.Converter;
/**
 * Exemplar
 * Copyright (c) 2004-2007, Stanford University
 * All rights reserved.
 *
 * <p>
 * This software is distributed under the <a href="http://hci.stanford.edu/research/copyright.txt">
 * BSD License</a>.
 * </p>
 * 
 * 
 * @author <a href="http://bjoern.org/">Bjoern Hartmann</a> ( bjoern(AT)stanford.edu )
 * @author Leith Abdulla
 */
public class ExemplarPreferencePage extends FieldEditorPreferencePage implements
		IWorkbenchPreferencePage {



	public ExemplarPreferencePage() {
		super(GRID);
		setPreferenceStore(Activator.getDefault().getPreferenceStore());
		setDescription("Exemplar Configuration");
	}

	@Override
	public void createFieldEditors() {
		addField(new RadioGroupFieldEditor(
				PreferenceConstants.P_INPUT_PROTOCOL,
			"Input:",
			1,
			new String[][] { { "d.tools / Wiring / Arduino via &COM port", PreferenceConstants.P_INPUT_OSCSERIAL },
						{ "SparkFun &WiTilt via COM port", PreferenceConstants.P_INPUT_WITILTSERIAL },
						{ "Make Controller via UDP port", PreferenceConstants.P_INPUT_MAKEUDP},
						{ "Generic OSC Messages via UDP port", PreferenceConstants.P_INPUT_UDP}
		}, getFieldEditorParent()));
		
		
		addField( new StringFieldEditor(PreferenceConstants.P_COMPORT, "COM Port Name:", getFieldEditorParent()));
		addField( new IntegerFieldEditor(PreferenceConstants.P_COMSPEED, "COM Port Speed:", getFieldEditorParent()));
		addField( new IntegerFieldEditor(PreferenceConstants.P_UDP_INPUT_PORT, "UDP Input Port:", getFieldEditorParent()));
		addField( new IntegerFieldEditor(PreferenceConstants.P_MAXA2DVAL, "Maximum A2D Value:", getFieldEditorParent()));
		//////////////////////////////////////////
		// OUTPUT
		addField(new RadioGroupFieldEditor(
				PreferenceConstants.P_OUTPUT_PROTOCOL,
			"Output:",
			1,
			new String[][] { { "XML over TCP for Flash", PreferenceConstants.P_OUTPUT_FLASH },
						{ "OSC over UDP", PreferenceConstants.P_OUTPUT_OSCUDP }
		}, getFieldEditorParent()));
		
		addField( new IntegerFieldEditor(PreferenceConstants.P_TCP_OUTPUT_PORT, "TCP Output Server Port:", getFieldEditorParent()));
		addField( new IntegerFieldEditor(PreferenceConstants.P_UDP_OUTPUT_PORT, "UDP Output Client Port:", getFieldEditorParent()));
		addField( new StringFieldEditor(PreferenceConstants.P_UDP_OUTPUT_IPADDRESS, "UDP Output Client IP address:", getFieldEditorParent()));
		/////////////////////////////////////////////
		addField(new RadioGroupFieldEditor(
				PreferenceConstants.P_ADJUSTREGION,
			"Smart Example Region Adjustment:",
			1,
			new String[][] { { "O&n", PreferenceConstants.P_ADJUSTON },
						{ "O&ff", PreferenceConstants.P_ADJUSTOFF }
		}, getFieldEditorParent()));
		
	}

	@Override
	public boolean performOk() {
		//make sure we perform the stores first
		boolean result = super.performOk();
		
		//then notify application of changes
		//first update the serial port 
		Activator.getDefault().reopenInputPort( getPreferenceStore().getString(PreferenceConstants.P_INPUT_PROTOCOL), 
				  getPreferenceStore().getString(PreferenceConstants.P_COMPORT));
		Activator.getDefault().registerInputListeners();
	
		//then update the serial port settings
		Converter.initialize();
		
		
		return result;
		
	}

	public void init(IWorkbench workbench) {
		// TODO Auto-generated method stub

	}

}
