package edu.stanford.hci.exemplar.widgets;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.Map;

import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseMoveListener;
import org.eclipse.swt.events.PaintEvent;
import org.eclipse.swt.events.PaintListener;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.widgets.Canvas;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;

import com.cloudgarden.resource.SWTResourceManager;

import edu.stanford.hci.exemplar.Activator;
import edu.stanford.hci.exemplar.BigViewMouseMode;
import edu.stanford.hci.exemplar.Event;
import edu.stanford.hci.exemplar.EventRegion;
import edu.stanford.hci.exemplar.EventRegionOrigin;
import edu.stanford.hci.exemplar.EventSet;
import edu.stanford.hci.exemplar.EventType;
import edu.stanford.hci.exemplar.ExemplarConstants;
import edu.stanford.hci.exemplar.PatternEventEvaluator;
import edu.stanford.hci.exemplar.Sensor;
import edu.stanford.hci.exemplar.ThresholdEventEvaluator;
import edu.stanford.hci.exemplar.UserMouseToolMode;
import edu.stanford.hci.exemplar.preferences.PreferenceConstants;


/**
* This code was edited or generated using CloudGarden's Jigloo
* SWT/Swing GUI Builder, which is free for non-commercial
* use. If Jigloo is being used commercially (ie, by a corporation,
* company or business for any purpose whatever) then you
* should purchase a license for each developer using Jigloo.
* Please visit www.cloudgarden.com for details.
* Use of Jigloo implies acceptance of these licensing terms.
* A COMMERCIAL LICENSE HAS NOT BEEN PURCHASED FOR
* THIS MACHINE, SO JIGLOO OR THIS CODE CANNOT BE USED
* LEGALLY FOR ANY CORPORATE OR COMMERCIAL PURPOSE.
*/
/**
 * Exemplar
 * Copyright (c) 2004-2007, Stanford University
 * All rights reserved.
 *
 * <p>
 * This software is distributed under the <a href="http://hci.stanford.edu/research/copyright.txt">
 * BSD License</a>.
 * </p>
 * 
 * 
 * @author <a href="http://bjoern.org/">Bjoern Hartmann</a> ( bjoern(AT)stanford.edu )
 * @author Leith Abdulla
 */
public class BigViewCanvas extends Canvas implements PropertyChangeListener {

	private static final int PIXEL_SNAP_SIZE = 3;
	public static final int EVENT_REGION_SYSTEM_ALPHA = 48;
	public static final int EVENT_REGION_ALPHA = 96;
	public static final int EVENT_REGION_HIGHLIGHT_ALPHA = 164;
	public static final int EVENT_REGION_MIN_PIXEL_MOVE = 5;
	private static final int THRESHOLD_MANIPULATOR_WIDTH = 50;
	private static final int THRESHOLD_MANIPULATOR_HEIGHT = 14;
	private Rectangle patternManipulator = new Rectangle(0,0,0,0);
	private Rectangle minThreshManipulator = new Rectangle(0,0,0,0);
	private Rectangle maxThreshManipulator = new Rectangle(0,0,0,0);
	private Rectangle minHysteresisManipulator = new Rectangle(0,0,0,0);
	private Rectangle maxHysteresisManipulator = new Rectangle(0,0,0,0);
	private Rectangle timeoutManipulator = new Rectangle(0,0,0,0);
	
	private DecimalFormat numberFormatter = new DecimalFormat("0.00");
	private Sensor sensor;
	private EventSet eventSet;
	int tFrom=0, tTo=100;

	private EventRegion activeRegion;
	private int drawStartTime;
	private BigViewMouseMode mouseMode = BigViewMouseMode.NONE;
	private MouseEvent lastMouseDownEvent;
	private MouseEvent lastMouseMoveEvent;
	private EventRegion tempRegion;
	private int mouseDownStart;
	private UserMouseToolMode userMouseMode = UserMouseToolMode.DRAW;
	private enum Manipulator {MINBOX,MAXBOX, PATTERNBOX, MINHYSTBOX, MAXHYSTBOX, TIMEOUTBOX};
	private float threshMin,threshMax;
	private int tempTimeout;
	public BigViewCanvas(Composite parent, int style) {
		super(parent, style);
		addPaintListener(new PaintListener() {
			public void paintControl(PaintEvent evt) {
				BigViewCanvas.this.paintControl(evt);
			}
		});
		addDisposeListener(new DisposeListener() {
	         public void widgetDisposed(DisposeEvent e) {
	                BigViewCanvas.this.widgetDisposed(e);
	         }
	     });		
		addMouseListener(new MouseAdapter() {
			public void mouseDown(MouseEvent e) {
				BigViewCanvas.this.mouseDown(e);
			}

			public void mouseUp(MouseEvent e) {
				BigViewCanvas.this.mouseUp(e);
			}
		});
		addMouseMoveListener(new MouseMoveListener() {
			public void mouseMove(MouseEvent e) {
				BigViewCanvas.this.mouseMove(e);
			}
		});
		if(Activator.getDefault()!=null) {
			Activator.getDefault().pcs.addPropertyChangeListener(ExemplarConstants.PROP_REDRAW, this);
			Activator.getDefault().pcs.addPropertyChangeListener(ExemplarConstants.PROP_USER_MOUSE_TOOL, this);
		}
		 
	}

	protected void initEventRegion()
	{
		mouseMode = BigViewMouseMode.DRAW;
		activeRegion = new EventRegion(drawStartTime,getTimeForPixel(mouseDownStart+1),null) ;
		activeRegion.setOrigin(EventRegionOrigin.USER_CREATED);
		Event evt = eventSet.getActiveEvent();
		evt.add(activeRegion);
	}
	private Rectangle getDrawingArea() {
		Rectangle r = getClientArea();
		return new Rectangle(r.x,r.y+4,r.width,r.height-8);
	}
	
	protected void mouseMove(MouseEvent me) {
		if(mouseMode.equals(BigViewMouseMode.INIT) && Math.abs(me.x - mouseDownStart) > EVENT_REGION_MIN_PIXEL_MOVE){
			initEventRegion();
		}
		else if(mouseMode.equals(BigViewMouseMode.DRAW)) {
			if(getTimeForPixel(me.x)>drawStartTime) {
				//drawing to the right
				activeRegion.setStartTime(drawStartTime);
				activeRegion.setEndTime(getTimeForPixel(me.x));
			} else {
				//drawing to the left
				activeRegion.setStartTime(getTimeForPixel(me.x));
				activeRegion.setEndTime(drawStartTime);
			}
			redraw();
		} else if(mouseMode.equals(BigViewMouseMode.MOVE)) {
			activeRegion.setStartTime(tempRegion.getStartTime()+getTimeOffsetForPixelOffset(me.x-lastMouseDownEvent.x));
			activeRegion.setEndTime(tempRegion.getEndTime()+getTimeOffsetForPixelOffset(me.x-lastMouseDownEvent.x));
			redraw();
		} else if(mouseMode.equals(BigViewMouseMode.RESIZE_LEFT)) {
			if(getTimeForPixel(me.x)>tempRegion.getEndTime()) {
				//drawing to the right
				activeRegion.setStartTime(tempRegion.getEndTime());
				activeRegion.setEndTime(getTimeForPixel(me.x));
			} else {
				//drawing to the left
				activeRegion.setStartTime(getTimeForPixel(me.x));
				activeRegion.setEndTime(tempRegion.getEndTime());
			}
			redraw();
		}  else if(mouseMode.equals(BigViewMouseMode.RESIZE_RIGHT)) {
			if(getTimeForPixel(me.x)<tempRegion.getStartTime()) {
				activeRegion.setStartTime(getTimeForPixel(me.x));
				activeRegion.setEndTime(tempRegion.getStartTime());
			} else {
				activeRegion.setStartTime(tempRegion.getStartTime());
				activeRegion.setEndTime(getTimeForPixel(me.x));
			}
			redraw();
		} else if (mouseMode.equals(BigViewMouseMode.DRAG_VIEW)) {
			//compute delta
			int delta;
			if(lastMouseMoveEvent==null) {
				delta= getTimeOffsetForPixelOffset(lastMouseDownEvent.x-me.x);
			} else {
				delta= getTimeOffsetForPixelOffset(lastMouseMoveEvent.x-me.x);
			}
			//update model
			//BigViewModel model = ((BigViewWithButtonsComposite)getParent()).getModel();
			//model.setStartTime(model.getStartTime()+delta);
			lastMouseMoveEvent=me;
			//hope that model updates view (=us)
			Activator.getDefault().pcs.firePropertyChange(ExemplarConstants.PROP_MOUSE_DRAGGING, null, delta);
		}else if(mouseMode.equals(BigViewMouseMode.DRAG_THRESH_MIN)) {
			int delta;
			if(lastMouseMoveEvent==null) {
				delta= (lastMouseDownEvent.y-me.y);
			} else {
				delta= (lastMouseMoveEvent.y-me.y);
			}
			
			//update min in event evaluator
			Event e = eventSet.getActiveEvent();
			ThresholdEventEvaluator tee = ((ThresholdEventEvaluator)e.getEvaluator(sensor.getOSCAddress()));
			tee.setMin(tee.getMin()+getValueForPixelOffset(delta));
			
			lastMouseMoveEvent=me;
		}else if(mouseMode.equals(BigViewMouseMode.DRAG_THRESH_MAX)) {
			int delta;
			if(lastMouseMoveEvent==null) {
				delta= (lastMouseDownEvent.y-me.y);
			} else {
				delta= (lastMouseMoveEvent.y-me.y);
			}
			//update min in event evaluator
			Event e = eventSet.getActiveEvent();
			ThresholdEventEvaluator tee = ((ThresholdEventEvaluator)e.getEvaluator(sensor.getOSCAddress()));
			tee.setMax(tee.getMax()+getValueForPixelOffset(delta));
			
			lastMouseMoveEvent=me;
		} else if(mouseMode.equals(BigViewMouseMode.DRAG_PATTERN_THRESH)) {
			int delta;
			if(lastMouseMoveEvent==null) {
				delta= (lastMouseDownEvent.y-me.y);
			} else {
				delta= (lastMouseMoveEvent.y-me.y);
			}
			Event e = eventSet.getActiveEvent();
			PatternEventEvaluator pee = ((PatternEventEvaluator)e.getEvaluator(sensor.getOSCAddress()));
			float deltaf = (float)delta/((getDrawingArea().height/2)/pee.getMedian());
			
			pee.setThreshold(pee.getThreshold()+deltaf);
			lastMouseMoveEvent=me;
		} else if (mouseMode.equals(BigViewMouseMode.DRAG_HYSTERESIS_MIN)) {
			int delta;
			if(lastMouseMoveEvent==null) {
				delta= (lastMouseDownEvent.y-me.y);
			} else {
				delta= (lastMouseMoveEvent.y-me.y);
			}
			//update min in event evaluator
			Event e = eventSet.getActiveEvent();
			ThresholdEventEvaluator tee = ((ThresholdEventEvaluator)e.getEvaluator(sensor.getOSCAddress()));
			tee.setMinHysteresis(Math.max(0.f, tee.getMinHysteresis()+getValueForPixelOffset(delta)));
			lastMouseMoveEvent=me;
		}else if (mouseMode.equals(BigViewMouseMode.DRAG_HYSTERESIS_MAX)) {
			int delta;
			if(lastMouseMoveEvent==null) {
				delta= (lastMouseDownEvent.y-me.y);
			} else {
				delta= (lastMouseMoveEvent.y-me.y);
			}
			//update max in event evaluator
			Event e = eventSet.getActiveEvent();
			ThresholdEventEvaluator tee = ((ThresholdEventEvaluator)e.getEvaluator(sensor.getOSCAddress()));
			tee.setMaxHysteresis(Math.max(0.f,tee.getMaxHysteresis()+getValueForPixelOffset(delta)));

			lastMouseMoveEvent=me;
		} else if (mouseMode.equals(BigViewMouseMode.DRAG_TIMEOUT)) {
			Event e = eventSet.getActiveEvent();
			ThresholdEventEvaluator tee = ((ThresholdEventEvaluator)e.getEvaluator(sensor.getOSCAddress()));
			tee.setTimeout(Math.max(0,tempTimeout+getTimeOffsetForPixelOffset(lastMouseDownEvent.x-me.x)));
		}
		
	}

	protected void mouseUp(MouseEvent e) {
		
		/*if(mouseMode.equals(BigViewMouseMode.INIT) && getTimeForPixel(e.x) > drawStartTime)
		{
			initEventRegion();
			mouseMove(e);
		}*/
		
		if(mouseMode.equals(BigViewMouseMode.DRAG_THRESH_MIN) || mouseMode.equals(BigViewMouseMode.DRAG_THRESH_MAX)) {
			//swap min/max if we moved one past the other
			Event es = eventSet.getActiveEvent();
			ThresholdEventEvaluator tee = ((ThresholdEventEvaluator)es.getEvaluator(sensor.getOSCAddress()));
			tee.checkAndSwap();
			//recalculate system regions without recalculating threshold values
			recalculate(false);
			eventSet.setModified(true);
		} else if(mouseMode.equals(BigViewMouseMode.DRAG_HYSTERESIS_MAX) || mouseMode.equals(BigViewMouseMode.DRAG_HYSTERESIS_MIN)) {
			recalculate(false);
			eventSet.setModified(true);
		} else if(mouseMode.equals(BigViewMouseMode.DRAG_TIMEOUT)) {
			recalculate(false);
			eventSet.setModified(true);
		} else if(mouseMode.equals(BigViewMouseMode.DRAG_PATTERN_THRESH)) {
			recalculate(false);
			eventSet.setModified(true);
		}else if(!mouseMode.equals(BigViewMouseMode.INIT) && !mouseMode.equals(BigViewMouseMode.DRAG_VIEW)){ //don't recalc if we didn't draw a new region
			recalculate(true);
			eventSet.setModified(true);
		
		}
		redraw();
		mouseMode = BigViewMouseMode.NONE;
		lastMouseMoveEvent=null;
	}
	
	private void recalculate(boolean relearn) {
		try {
			if(Activator.getDefault().getPreferenceStore().getString(PreferenceConstants.P_ADJUSTREGION).equals(PreferenceConstants.P_ADJUSTON)) {
				eventSet.getActiveEvent().adjustRegion(activeRegion);
			}
			eventSet.getActiveEvent().recalculate(relearn);
		}catch(NullPointerException e){
			
		};
	}
	protected void mouseDown(MouseEvent e) {
		
		lastMouseDownEvent	= e;
		
		Activator.getDefault().setSingleActiveSensor(sensor.getOSCAddress());
		
		
		
		if(userMouseMode.equals(UserMouseToolMode.DRAW)) {
			
			
			
			Event evt = eventSet.getActiveEvent();
			 
			if(e.button==1 && evt.getType(sensor.getOSCAddress()).equals(EventType.THRESHOLD)) {
				// first check if we are moving the min/max bars
				if(isManipulatorUnderMouse(Manipulator.MINBOX,e)) {
					mouseMode = BigViewMouseMode.DRAG_THRESH_MIN;
					return;
				} else if(isManipulatorUnderMouse(Manipulator.MAXBOX,e)) {
					mouseMode = BigViewMouseMode.DRAG_THRESH_MAX;
					return;
				}
				//or the hysteresis manipulator
				else if (isManipulatorUnderMouse(Manipulator.MINHYSTBOX, e)) {
					mouseMode = BigViewMouseMode.DRAG_HYSTERESIS_MIN;
					return;
				}else if (isManipulatorUnderMouse(Manipulator.MAXHYSTBOX, e)) {
					mouseMode = BigViewMouseMode.DRAG_HYSTERESIS_MAX;
					return;
				} else if(isManipulatorUnderMouse(Manipulator.TIMEOUTBOX, e)) {
					mouseMode = BigViewMouseMode.DRAG_TIMEOUT;
					Event ev = eventSet.getActiveEvent();
					if(ev.getEvaluator(sensor.getOSCAddress()) instanceof ThresholdEventEvaluator) {
						ThresholdEventEvaluator tee = ((ThresholdEventEvaluator)ev.getEvaluator(sensor.getOSCAddress()));
						tempTimeout = tee.getTimeout();
					} else {
						tempTimeout=0;
					}
					return;
				}
				
			}
			//
			//or if we're moving the pattern threshold bar
			if(e.button==1 && evt.getType(sensor.getOSCAddress()).equals(EventType.PATTERN)) {
				if(isManipulatorUnderMouse(Manipulator.PATTERNBOX,e)) {
					mouseMode = BigViewMouseMode.DRAG_PATTERN_THRESH;
					return;
				}
			}
			// if no, then check if we're modifying an existing event
			if(evt!=null) {
				// find EventRegion under mouse
				EventPair p = findEventRegionUnderMouse(e);
				if(p!=null && p.region.getOrigin().equals(EventRegionOrigin.USER_CREATED)) {
					// if there is one, MOVE or RESIZE or DELETE it
					if(e.button==1) {
						//left button: MOVE or RESIZE
						activeRegion = p.region;
						tempRegion = new EventRegion(p.region);
						
						if(e.x<=(getPixelForTime(p.region.getStartTime()))+PIXEL_SNAP_SIZE) {
							mouseMode = BigViewMouseMode.RESIZE_LEFT;
						} else if (e.x>=(getPixelForTime(p.region.getEndTime()))-PIXEL_SNAP_SIZE) {
							mouseMode = BigViewMouseMode.RESIZE_RIGHT;
						} else {
							mouseMode = BigViewMouseMode.MOVE;
						}
					} else {
						//middle or right button:DELETE
						p.event.removeRegion(p.region);
						activeRegion=null;
						mouseMode = BigViewMouseMode.NONE;
						redraw();
					}
					
				} else {
					if(e.button==1) {
						// if there isn't DRAW a new one on left button
						mouseMode = BigViewMouseMode.INIT;
						mouseDownStart 	= e.x;
						drawStartTime = getTimeForPixel(mouseDownStart);
						/*
						activeRegion = new EventRegion(getTimeForPixel(e.x),getTimeForPixel(e.x+1)) ;
						activeRegion.setOrigin(EventRegionOrigin.USER_CREATED);
						evt.add(activeRegion);
						*/
					}
				}
			}
		} else if(userMouseMode.equals(UserMouseToolMode.DRAG)) {
			if(e.button==1) {
				// if there isn't DRAW a new one on left button
				mouseMode = BigViewMouseMode.DRAG_VIEW;
				mouseDownStart 	= e.x;
				drawStartTime = getTimeForPixel(mouseDownStart);
			}
		}
		
	}
	private boolean isManipulatorUnderMouse(Manipulator m, MouseEvent e) {
		if(Manipulator.MINBOX.equals(m)) {
			return minThreshManipulator.contains(new Point(e.x,e.y));
		} else if (Manipulator.MAXBOX.equals(m)) {
			return maxThreshManipulator.contains(new Point(e.x,e.y));
		} else if (Manipulator.PATTERNBOX.equals(m)) {
			return patternManipulator.contains(new Point(e.x,e.y));
		} else if (Manipulator.MINHYSTBOX.equals(m)) {
			return minHysteresisManipulator.contains(new Point(e.x, e.y));
		} else if (Manipulator.MAXHYSTBOX.equals(m)) {
			return maxHysteresisManipulator.contains(new Point(e.x, e.y));
		} else if (Manipulator.TIMEOUTBOX.equals(m)) {
			return timeoutManipulator.contains(new Point(e.x, e.y));
		}
		return false;
	}

	/**
	 * Search through all event regions and return one region whose figure was clicked on with the mouse. If multiple match, which one is returned is not specified. Presumably the one added first. This method only matches user-created regions.
	 * @param me MouseEvent that contains click coordinates for search
	 * @return null if no match found, other wise the event region object clicked.
	 */
	private EventPair findEventRegionUnderMouse(MouseEvent me) {
		int t = getTimeForPixel(me.x);
		for(Event e:eventSet.getEvents()) {
			for(EventRegion r:e.getRegions()) {
				if(r.isPointInRegion(t) && r.getOrigin().equals(EventRegionOrigin.USER_CREATED)) {
					
					return new EventPair(e,r);
				}
			}
		}
		return null;
	}

	protected void widgetDisposed(DisposeEvent e) {
		// TODO Auto-generated method stub
	}

	protected void paintControl(PaintEvent evt) {
		
		paintEventOverlays(evt);
		paintTimeMarks(evt);
		paintSensorData(evt);
		paintEventDecorations(evt);
		paintTimeoutBar(evt);
	}

	private void paintTimeoutBar(PaintEvent evt) {
		Event e = eventSet.getActiveEvent();
			if(e!=null && e.getType(sensor.getOSCAddress()).equals(EventType.THRESHOLD)) {
				ThresholdEventEvaluator tee = ((ThresholdEventEvaluator)e.getEvaluator(sensor.getOSCAddress()));	
					evt.gc.setBackground(SWTResourceManager.getColor(128, 128, 128));
					evt.gc.setAlpha(0x40);
					int w = getPixelOffsetForTimeOffset(tee.getTimeout());
					evt.gc.fillRectangle(getClientArea().width-w,0,w,getClientArea().height);
					
					evt.gc.setForeground(SWTResourceManager.getColor(128, 128, 128));
					evt.gc.setAlpha(0xFF);
					//draw line down
					evt.gc.drawLine(getClientArea().width-w, 0, getClientArea().width-w, getClientArea().height);
					//draw manipulator
					timeoutManipulator.x=getClientArea().width-(w+46);
					timeoutManipulator.y=getClientArea().height-15;
					timeoutManipulator.width=46;
					timeoutManipulator.height=14;
				
					evt.gc.setBackground(SWTResourceManager.getColor(255,255,255));
					evt.gc.fillRectangle(timeoutManipulator);
					evt.gc.drawText("timeout", timeoutManipulator.x+2, timeoutManipulator.y);
					evt.gc.drawRectangle(timeoutManipulator);
			}
	}

	/**
	 * Paint a regular time marks to give a visual indication of how much time each pixel represents
	 * @param evt
	 */
	private void paintTimeMarks(PaintEvent evt) {
		evt.gc.setForeground(SWTResourceManager.getColor(128, 128, 128));
		evt.gc.setAlpha(0x40);
		evt.gc.setLineWidth(0);
		for( int t = (int)Math.ceil(((double)tFrom)/((double)50)); t<=(int)Math.floor(((double)tTo)/((double)50)); t++) {
			int x = getPixelForTime(t*50);
			if(t%10==0) {
				evt.gc.drawLine(x, getClientArea().y, x, getClientArea().y+15);
			} else if(t%5==0) {
				evt.gc.drawLine(x, getClientArea().y, x, getClientArea().y+10);
			} else {	
				evt.gc.drawLine(x, getClientArea().y, x, getClientArea().y+5);
			}
			
		}
		
	}

	private void paintEventDecorations(PaintEvent evt) {
		if(sensor!=null)
			//evt.gc.drawText(sensor.getOSCAddress()+" "+Integer.toString(tFrom)+"-"+Integer.toString(tTo)+": "+sensor.getBufferSize(), 10, 10);
			evt.gc.drawText(sensor.getOSCAddress(), 10, 10);
		Event e = eventSet.getActiveEvent();
		if(e!=null && e.getType(sensor.getOSCAddress()).equals(EventType.THRESHOLD)) {
			
			// draw threshold lines
			ThresholdEventEvaluator tee = ((ThresholdEventEvaluator)e.getEvaluator(sensor.getOSCAddress()));
			
			// min + max values
			threshMin = tee.getMin();
			threshMax = tee.getMax();
			
			
			evt.gc.setForeground(SWTResourceManager.getColor(e.getColor().red, e.getColor().green, e.getColor().blue));
			
			evt.gc.setAlpha(0xFF);
			evt.gc.setLineWidth(0);
			// draw the min threshold line(s)
			if(tee.getMinHysteresis()>0.f) {
				evt.gc.drawLine(getDrawingArea().x+64, getPixelForValue(threshMin-tee.getMinHysteresis()), getDrawingArea().x+getDrawingArea().width, getPixelForValue(threshMin-tee.getMinHysteresis()));
				evt.gc.drawLine(getDrawingArea().x+64, getPixelForValue(threshMin+tee.getMinHysteresis()), getDrawingArea().x+getDrawingArea().width, getPixelForValue(threshMin+tee.getMinHysteresis()));
				evt.gc.drawLine(getDrawingArea().x+64, getPixelForValue(threshMin-tee.getMinHysteresis()), getDrawingArea().x+64, getPixelForValue(threshMin+tee.getMinHysteresis()));
			} else {
				evt.gc.drawLine(getDrawingArea().x+64, getPixelForValue(threshMin), getDrawingArea().x+getDrawingArea().width, getPixelForValue(threshMin));
			}
			// draw the max threshold line(s)
			if(tee.getMaxHysteresis()>0.f) {
				evt.gc.drawLine(getDrawingArea().x+64, getPixelForValue(threshMax-tee.getMaxHysteresis()), getDrawingArea().x+getDrawingArea().width, getPixelForValue(threshMax-tee.getMaxHysteresis()));
				evt.gc.drawLine(getDrawingArea().x+64, getPixelForValue(threshMax+tee.getMaxHysteresis()), getDrawingArea().x+getDrawingArea().width, getPixelForValue(threshMax+tee.getMaxHysteresis()));
				evt.gc.drawLine(getDrawingArea().x+64, getPixelForValue(threshMax-tee.getMaxHysteresis()), getDrawingArea().x+64, getPixelForValue(threshMax+tee.getMaxHysteresis()));
			} else {
				evt.gc.drawLine(getDrawingArea().x+64, getPixelForValue(threshMax), getDrawingArea().x+getDrawingArea().width, getPixelForValue(threshMax));
			}
			
			// draw the min and max manipulators
			evt.gc.setLineWidth(0);
			evt.gc.setAlpha(0xFF);
			evt.gc.setBackground(SWTResourceManager.getColor(255,255,255));
			evt.gc.fillRectangle(minThreshManipulator);
			evt.gc.fillRectangle(maxThreshManipulator);
			
			evt.gc.drawText("min "+numberFormatter.format(threshMin), getDrawingArea().x+2, getPixelForValue(threshMin)-7);
			evt.gc.drawText("max "+numberFormatter.format(threshMax), getDrawingArea().x+2, getPixelForValue(threshMax)-7);
			minThreshManipulator.x=0;
			minThreshManipulator.y=getPixelForValue(threshMin)-7;
			minThreshManipulator.width=50;
			minThreshManipulator.height=14;
			maxThreshManipulator.x=0;
			maxThreshManipulator.y=getPixelForValue(threshMax)-7;
			maxThreshManipulator.width=50;
			maxThreshManipulator.height=14;

			evt.gc.drawRectangle(minThreshManipulator);
			evt.gc.drawRectangle(maxThreshManipulator);
			
			// draw the hysteresis manipulators
			minHysteresisManipulator.x=50;
			minHysteresisManipulator.width=14;
			minHysteresisManipulator.height=14;
			minHysteresisManipulator.y = getPixelForValue(threshMin)-7;
			maxHysteresisManipulator.x=50;
			maxHysteresisManipulator.width=14;
			maxHysteresisManipulator.height=14;
			maxHysteresisManipulator.y = getPixelForValue(threshMax)-7;
			evt.gc.setBackground(SWTResourceManager.getColor(255,255,255));
			evt.gc.fillRectangle(minHysteresisManipulator);
			evt.gc.fillRectangle(maxHysteresisManipulator);
			evt.gc.drawText("H", 53, getPixelForValue(threshMin)-7);
			evt.gc.drawText("H", 53, getPixelForValue(threshMax)-7);
			evt.gc.drawRectangle(minHysteresisManipulator);
			evt.gc.drawRectangle(maxHysteresisManipulator);
		}
		else if(e!=null && e.getType(sensor.getOSCAddress()).equals(EventType.PATTERN))
		{
			PatternEventEvaluator v = (PatternEventEvaluator)e.getEvaluator(sensor.getOSCAddress());
			Map <Integer, Float> error = v.getPatternError();
			
			float thresh = v.getThreshold();
			float median = v.getMedian();
			int time;
			//int points[] = new int[getDrawingArea().width*2];
			int height = getDrawingArea().y + getDrawingArea().height;
			
			ArrayList<Integer> points = new ArrayList<Integer>();
			int points2[];
			Float f;
			
			float midpoint 	= (getDrawingArea().height)/2;
			float scalar 	= midpoint / median;
			
			
			for(int i = getDrawingArea().x, j=0; i < getDrawingArea().x + getDrawingArea().width; i++, j+=2)
			{
				time = getTimeForPixel(i);
				if(error.containsKey(time))
				{
					points.add(i);
					f = error.get(time)*scalar;
					points.add(height - Math.min(f.intValue(), height));
				}	
			}
			
			//points.add(getDrawingArea().x + getDrawingArea().width);
			//points.add(height - e);
			
			points2 = new int[points.size()];
			
			for(int i = 0; i < points.size(); i++)
			{
				points2[i] = points.get(i);
			}
			
			evt.gc.setForeground(SWTResourceManager.getColor(e.getColor().red, e.getColor().green, e.getColor().blue));
			evt.gc.setLineWidth(0);
			evt.gc.setAlpha(0xFF);
			evt.gc.drawPolyline(points2);
			evt.gc.setLineWidth(2);
			evt.gc.drawLine(getDrawingArea().x+50, height - (int) (thresh*scalar), getDrawingArea().x + getDrawingArea().width, height - (int) (thresh*scalar));
			patternManipulator.x=0;
			patternManipulator.width=50;
			patternManipulator.y = height - (int) (thresh*scalar) -7;
			patternManipulator.height=14;
			evt.gc.drawText("Thresh", 2, height - (int) (thresh*scalar)-7);
			evt.gc.drawRectangle(patternManipulator);
		}
	}

	private void paintEventOverlays(PaintEvent evt) {
		if(eventSet==null) return;
		Point p; 
		//TODO: SLOW!
		for(Event e:eventSet.getEvents()) {
			synchronized(e) {
			for(EventRegion r:e.getRegions()) {
				if( (p=r.getVisibleRange(tFrom,tTo))!=null) {
					if(r.getOrigin().equals(EventRegionOrigin.USER_CREATED) ) {//&& r.getAddress().equals(sensor.getOSCAddress())
					// user regions
						if(r==activeRegion) {
							// draw foreground
							evt.gc.setForeground(SWTResourceManager.getColor(e.getColor().red, e.getColor().green, e.getColor().blue));
							evt.gc.setLineWidth(2);
							evt.gc.setAlpha(0xFF);
							evt.gc.drawRectangle(getPixelForTime(p.x), 0, getPixelForTime(p.y)-getPixelForTime(p.x), this.getClientArea().height);
							 // draw background
							evt.gc.setBackground(SWTResourceManager.getColor(e.getColor().red, e.getColor().green, e.getColor().blue)); 
							evt.gc.setAlpha(EVENT_REGION_HIGHLIGHT_ALPHA);
							evt.gc.fillRectangle(getPixelForTime(p.x), 0, getPixelForTime(p.y)-getPixelForTime(p.x), this.getClientArea().height);
							evt.gc.setForeground(SWTResourceManager.getColor(0,0,0));
							evt.gc.drawText("U", getPixelForTime(p.x), 2);
							//evt.gc.drawText("Active region: ["+r.getStartTime()+"-"+r.getEndTime()+"]", 5, this.getClientArea().height-10);
						} else {
							
							// draw foreground
							evt.gc.setForeground(SWTResourceManager.getColor(e.getColor().red, e.getColor().green, e.getColor().blue));
							evt.gc.setLineWidth(0);
							evt.gc.setAlpha(0xA0);
							evt.gc.drawRectangle(getPixelForTime(p.x), 0, getPixelForTime(p.y)-getPixelForTime(p.x), this.getClientArea().height);
							
							// draw background
							evt.gc.setBackground(SWTResourceManager.getColor(e.getColor().red, e.getColor().green, e.getColor().blue));
							evt.gc.setAlpha(EVENT_REGION_ALPHA);
							evt.gc.fillRectangle(getPixelForTime(p.x), 0, getPixelForTime(p.y)-getPixelForTime(p.x), this.getClientArea().height);
							
							evt.gc.setForeground(SWTResourceManager.getColor(0,0,0));
							evt.gc.drawText("U", getPixelForTime(p.x), 2);
						}
						
					} else {
						if(sensor.getOSCAddress().equals(r.getAddress())) {
							// draw background only
							
							
							evt.gc.setBackground(SWTResourceManager.getColor(e.getColor().red, e.getColor().green, e.getColor().blue));
							evt.gc.setAlpha(EVENT_REGION_SYSTEM_ALPHA);
							evt.gc.fillRectangle(getPixelForTime(p.x), 0, getPixelForTime(p.y)-getPixelForTime(p.x), this.getClientArea().height);
							
							evt.gc.setForeground(SWTResourceManager.getColor(0,0,0));
							evt.gc.setAlpha(0x80);
							evt.gc.drawText("S", getPixelForTime(p.x), 2);
						}
					}
				 }				
			}
		}
		}
	}

	/// returns the x-pixel coordinate for a given time value between tFrom and tTo
	private int getPixelForTime(int x) {
		Rectangle r = this.getDrawingArea();
		int delta = tTo-tFrom;
		return r.x+((Float)(
				(Float)(
				(
						((Integer)(x-tFrom)).floatValue()/
						((Integer)delta).floatValue()
				)
				)*((Integer)r.width).floatValue())).intValue();//x
	}
	/// returns Y-coordinate for a given Float sensor value in range (0..1)
	private int getPixelForValue(Float val) {
		Rectangle r = this.getDrawingArea();
		return r.y+
		((Float)((1.f-val)*((Integer)(r.height)).floatValue())).intValue();
	}
	private float getValueForPixelOffset(int pixel) {
		Rectangle r = this.getDrawingArea();
		return ((float)pixel/(float)r.height);
	}
	private void paintSensorData(PaintEvent evt) {
		if(sensor==null) return;
		
		Rectangle r = this.getDrawingArea();
		
		
		if(sensor.equals(Activator.getDefault().getSingleActiveSensor()) && Activator.getDefault().getActiveSensors().size() > 1)
		{
			evt.gc.setForeground(SWTResourceManager.getColor(255, 0, 0));
			evt.gc.setLineWidth(10);
			evt.gc.drawLine(r.x, r.y, r.x, r.height);
			evt.gc.drawLine(r.x, r.y, r.width, r.y);
			evt.gc.drawLine(r.width, r.y, r.width, r.height);
			evt.gc.drawLine(r.x, r.height, r.width, r.height);
		}
		
		//for current Sensor,
		//iterate tFrom->tTo
		//draw polyPoints
		evt.gc.setForeground(SWTResourceManager.getColor(0,0,0));
		evt.gc.setAlpha(0xFF);
		evt.gc.setLineWidth(0);
		
		
		
		
		/* drawing routine one: by values 
		 * draws O(tTo-tFrom) line segments - SLOW when zoomed out
		 */
		/*
		int[] pointArray = new int[2*((tTo-tFrom)+3)];
		
		

		int pNum=0;
		
		for(int t=tFrom; t<=tTo; t++) {
			
			pointArray[2*pNum] = getPixelForTime(t); //x coordinate
			pointArray[2*pNum+1] = getPixelForValue(sensor.getValueForTime(t)); //y coordinate
			pNum++;
		}
		pointArray[2*pNum] =r.x+r.width;
		pointArray[2*pNum+1] =r.y+r.height;
		pointArray[2*pNum+2] =r.x;
		pointArray[2*pNum+3] =r.y+r.height;
		evt.gc.drawPolyline(pointArray);
		*/
		/* drawing rountine 2 - by pixels 
		 * draws O(r.width) line segments
		 * fast, but has visual flicker
		 * */
		
		int[] pointArray2 = new int[4*r.width+4];
		
		pointArray2[0] = 0;
		pointArray2[1] = r.height;
		
		for(int x=0;x<r.width;x++) {
			int sliceFrom = getTimeForPixel(x)-(int)(((float)(tTo-tFrom)/r.width)/2.0);
			int sliceTo = getTimeForPixel(x)+(int)(((float)(tTo-tFrom)/r.width)/2.0);
			int x1 = r.x+x; //x coord
			int y1 =  getPixelForValue(sensor.getMinValueForTimeSlice(sliceFrom, sliceTo));
			int x2 = x1;
			int y2 = getPixelForValue(sensor.getMaxValueForTimeSlice(sliceFrom,sliceTo));
			pointArray2[4*x+2] = x1;
			pointArray2[4*x+3] = y1;
			pointArray2[4*x+4] = x2;
			pointArray2[4*x+5] = y2;
			//evt.gc.drawLine(x1, y1, x2, y2);
		}
		
		pointArray2[4*r.width+2] = r.width;
		pointArray2[4*r.width+3] = r.height;
		
		evt.gc.setBackground(SWTResourceManager.getColor(0,0,0));
		evt.gc.drawPolyline(pointArray2);
		//evt.gc.fillPolygon(pointArray2);
		evt.gc.setBackground(SWTResourceManager.getColor(255,255,255));
	}

	

	private Integer getTimeForPixel(int x) {
		Rectangle r = this.getDrawingArea();
		return tFrom+(int)(((float)x)/((float)r.width)*(tTo-tFrom));
	}
	private int getTimeOffsetForPixelOffset(int x) {
		return (int)(((float)x)/((float)this.getDrawingArea().width)*(tTo-tFrom));
	}
	private int getPixelOffsetForTimeOffset(int x) {
		return (int)(((float)x)/(tTo-tFrom)*((float)this.getDrawingArea().width));
	}

	public void setSensor(Sensor sensor) {
		this.sensor = sensor;
		this.eventSet = Activator.getDefault().createEventSetForOscAddressList(Activator.getDefault().getActiveSensors()); //can't get = still null
		//this.eventSet = Activator.getDefault().getEventSetForOscAddress(sensor.getOSCAddress());
	}

	public void propertyChange(PropertyChangeEvent evt) {
//		REDRAW GRAPHICS
		if(ExemplarConstants.PROP_REDRAW.equals(evt.getPropertyName())) {
			redrawInUIThread();
		} else if(ExemplarConstants.PROP_USER_MOUSE_TOOL.equals(evt.getPropertyName())) {
			userMouseMode  = (UserMouseToolMode)evt.getNewValue();
		}
	}

	public void redrawInUIThread() {
		//redraw asynchronously in the UI thread
		Display d = Activator.getDefault().getWorkbench().getDisplay();
		if(!d.isDisposed()) {
			try {
				Activator.getDefault().getWorkbench().getDisplay().asyncExec (new Runnable () {
					public void run () {
						if(!isDisposed())
							redraw();
					}
				});
			} catch (NullPointerException e) {
				//may throw an exception while we're shutting down.
			}
		}
	}
	
	public void setStartTime(int from) {
		tFrom = from;
	}
	public void setEndTime(int to) {
		tTo = to;
	}
	
	private class EventPair {
		public Event event;
		public EventRegion region;
		public EventPair(Event event, EventRegion region) {
			super();
			this.event = event;
			this.region = region;
		}
		
	}

	public void setActiveRegion(EventRegion region) {
		activeRegion=region;
	}
}
