package edu.stanford.hci.exemplar.widgets;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.Timer;
import java.util.TimerTask;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.PaintEvent;
import org.eclipse.swt.events.PaintListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.widgets.Canvas;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;

import com.cloudgarden.resource.SWTResourceManager;

import edu.stanford.hci.exemplar.Activator;
import edu.stanford.hci.exemplar.ExemplarConstants;
import edu.stanford.hci.exemplar.RingBuffer;


/**
* This code was edited or generated using CloudGarden's Jigloo
* SWT/Swing GUI Builder, which is free for non-commercial
* use. If Jigloo is being used commercially (ie, by a corporation,
* company or business for any purpose whatever) then you
* should purchase a license for each developer using Jigloo.
* Please visit www.cloudgarden.com for details.
* Use of Jigloo implies acceptance of these licensing terms.
* A COMMERCIAL LICENSE HAS NOT BEEN PURCHASED FOR
* THIS MACHINE, SO JIGLOO OR THIS CODE CANNOT BE USED
* LEGALLY FOR ANY CORPORATE OR COMMERCIAL PURPOSE.
*/
/**
 * Exemplar
 * Copyright (c) 2004-2007, Stanford University
 * All rights reserved.
 *
 * <p>
 * This software is distributed under the <a href="http://hci.stanford.edu/research/copyright.txt">
 * BSD License</a>.
 * </p>
 * 
 * 
 * @author <a href="http://bjoern.org/">Bjoern Hartmann</a> ( bjoern(AT)stanford.edu )
 * @author Leith Abdulla
 */
public class SensorDataCanvas extends Canvas implements PropertyChangeListener {

	{
		//Register as a resource user - SWTResourceManager will
		//handle the obtaining and disposing of resources
		SWTResourceManager.registerResourceUser(this);
	}


	
	private Composite parent;
	private Color white,black;
	private RingBuffer buffer;
	private static final int bufferSize=100;
	private int count = 1;
	private float avg, last = 0;
	private Timer timer = new Timer();
	private int timerCount = 0;
	
	private String OSCAddress;
	private int alpha=0;
	
	public SensorDataCanvas(Composite parent, int style,String OSCAddress) {
		// parent is the group
		super(parent, style);
		this.OSCAddress=OSCAddress;
		this.parent = parent;
		init();
	}
	
	private void init() {
		white = new Color(null, 255, 255, 255);
		black = new Color(null, 0,0,0);
		buffer = new RingBuffer(bufferSize);
	    this.setBackground(SWTResourceManager.getColor(255, 255, 255));
		addPaintListener(new PaintListener() {
			public void paintControl(PaintEvent evt) {
				SensorDataCanvas.this.paintControl(evt);
			}
		});
		
		/*addMouseListener(new MouseAdapter() {
			public void mouseDown(MouseEvent evt) {
				AllSensorsComposite.makeSelection((OneSensorComposite)parent.getParent(),(evt.stateMask & SWT.CTRL)!=0);
				
			}
		});
		*/
		addDisposeListener(new DisposeListener() {
	         public void widgetDisposed(DisposeEvent e) {
	                SensorDataCanvas.this.widgetDisposed(e);
	         }
	     });

		if(Activator.getDefault()!=null) {
			Activator.getDefault().pcs.addPropertyChangeListener(OSCAddress, this);
			Activator.getDefault().pcs.addPropertyChangeListener(ExemplarConstants.PROP_REDRAW, this);
		}
		
	}


	protected void widgetDisposed(DisposeEvent e) {
		//dispose of resources that we allocated
		white.dispose();
		black.dispose();
		//and stop listening to property changes
		Activator.getDefault().pcs.removePropertyChangeListener(OSCAddress,this);
		Activator.getDefault().pcs.removePropertyChangeListener(ExemplarConstants.PROP_REDRAW, this);
	}

	// draw our graphics
	protected void paintControl(PaintEvent evt) {
		Rectangle r = this.getClientArea();

		//paint the "interesting event hightlight
		if(alpha>0) {
			evt.gc.setBackground(SWTResourceManager.getColor(255, 128, 128));
			evt.gc.setAlpha(alpha);
			evt.gc.fillRectangle(this.getClientArea());
		}
		
		//paint the data
		evt.gc.setAlpha(0xFF);
			
		int[] pointArray = new int[2*bufferSize+4];
		
		pointArray[0] = r.width;
		pointArray[1] = r.height;
		
		for(int i=0; i<bufferSize; i++) {
			/*evt.gc.drawLine(r.x+r.width-i,
					r.y+r.height,
					r.x+r.width-i,
					r.y + ((Float)((1.f-buffer.get(i))*((Integer)(r.height)).floatValue())).intValue()
					);*/
			pointArray[2*i+2]=r.x+r.width-i;
			pointArray[2*i+3]=r.y + ((Float)((1.f-buffer.get(i))*((Integer)(r.height)).floatValue())).intValue();
		}
		pointArray[2*bufferSize+2] = 0;
		pointArray[2*bufferSize+3] = r.height;
		
		evt.gc.setBackground(black);
		evt.gc.drawPolyline(pointArray);
		evt.gc.fillPolygon(pointArray);

		//paint the osc address on top
		evt.gc.setForeground(SWTResourceManager.getColor(0,0,0));
		evt.gc.setBackground(SWTResourceManager.getColor(255,255,255));
		evt.gc.drawText(OSCAddress, 5, 5);
	}

	private class interestingEvent extends TimerTask {
	   	private static final int steps = 20;
		public void run() {
	   		if(timerCount <= steps)
	   		{
	   			SensorDataCanvas.this.alpha = ((int) (255.f *((float)(steps-timerCount))/((float)steps)));
	   			timerCount++;
	   		}
	   		else
	   		{
	   			SensorDataCanvas.this.alpha=0;
	   			timerCount = 0;
	   			this.cancel();
	   		}   		
	    }
	 }
	
	// data aquisition and redraw requests communicate with our object through 
	// property change events
	public void propertyChange(PropertyChangeEvent evt) {
		//UPDATE BUFFER DATA if it is for us
		if(OSCAddress.equals(evt.getPropertyName())) {
			
			if(Math.abs((last - (Float)evt.getNewValue())) > .3)
			{
				if(timerCount == 0)
				{	
					timer.schedule(new interestingEvent(), 0, 50);
				}
				timerCount = 1;
				last = (Float)evt.getNewValue();
			}
			
			if((count % 3) == 2)
			{
				buffer.add(((Float)evt.getNewValue() + avg)/3);
			}
			if((count % 3) == 1)
			{
				avg += (Float)evt.getNewValue();
			}
			else
				avg = (Float)evt.getNewValue();
			
			if((count %20) == 0)
				last	  = (Float)evt.getNewValue();

			
			count = ((count+1)%bufferSize);
		}
		//REDRAW GRAPHICS
		if(ExemplarConstants.PROP_REDRAW.equals(evt.getPropertyName())) {
			//redraw asynchronously in the UI thread
			Display d = Activator.getDefault().getWorkbench().getDisplay();
			if(!d.isDisposed()) {
				d.asyncExec (new Runnable () {
					public void run () {
						if(!isDisposed())
							redraw();
					}
				});
			}
		}
	}
}
